
package ca.virology.japafi.tests;

import java.util.BitSet;
import java.util.List;

import ca.virology.japafi.algorithms.*;
import ca.virology.japafi.utils.*;


public class TestAPBTPattern
{
	/**************************************************************

	 * MAIN

	 *************************************************************/	
	 /**
	  * Tests the performance of APBTBoolean 
	  * when only start positions of pattern in the first string are required
	  */		

		public static void main(String [] args)
		{
			
			String file1=null;		
			String file2=null;
			int maxDiff=0;
			int minLen=0;			
			
			try
			{
				file1=args[0];		
				file2=args[1];
				
				minLen=Integer.parseInt(args[2]);
				maxDiff=Integer.parseInt(args[3]);
				
				
			}
			catch(Exception e)
			{
				System.out.println("Usage: \n" + 
						"java -Xmx512M -Xms512m -classpath classes ca.virology.japafi.algorithms.TestAPBTPattern \\ \n" +
	            		"<filename1> <filename2> \n" +
	            		"<minLength> <maxDifferences> " );
				System.exit(1);
			}	


			
			System.out.println("APBTBoolean algorithm");					
			
			
			SequenceFileReader reader=new SequenceFileReader(file1);
			String firstString=reader.getSequence();
			char [] seq1=firstString.toCharArray();
			BitSet oldStartPositions=new BitSet(seq1.length);
			oldStartPositions.set(0,seq1.length-1);
			
			reader=new SequenceFileReader(file2);
			char [] seq2=reader.getSequence().toCharArray();
			
			APBTBoolean algorithm=new APBTBoolean(oldStartPositions,seq1,seq2,
					minLen,maxDiff);
			
			long start=System.currentTimeMillis();
			algorithm.process();
			
			long howlong=System.currentTimeMillis()-start;
			System.out.println("Processed by APBT boolean in "+howlong+" ms.");
			BitSet newStartPositions=algorithm.getNewStartPositions();
			
			List startPosAsList=Util.bitSetToPositions(newStartPositions,seq1.length);
			System.out.println("Produced output size="+startPosAsList.size());
			
			System.out.println("The solutions are: "+startPosAsList);
			
			start=System.currentTimeMillis();
			
			for(int i=0;i<startPosAsList.size();i++)
			{
				int startPos=((Integer)startPosAsList.get(i)).intValue();
				char [] pattern=firstString.substring(startPos, Math.min(startPos+minLen+maxDiff,firstString.length())).toCharArray();
				APBTPattern patternB=new APBTPattern(seq2,pattern,minLen, maxDiff);
				patternB.process();
				System.out.println(startPos+": "+patternB.getSolutions());
			}
			
			System.out.println("Collected "+startPosAsList.size()
					+" patterns by APBT pattern in "
					+(System.currentTimeMillis()-start)+" ms.");
		}			

}
